//---------------------------------------------------------------------------
// GlyphConvertTask.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "GlyphConvertCommand.h"
#include "GlyphConvertTask.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace FullGlyph { namespace GlyphConvertTask {
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString DateTimeFormat = "yyyy-mm-dd hh:nn:ss";
const AnsiString ErrorPrefix = "[Error] ";
const AnsiString WarningPrefix = "[Warning] ";
const AnsiString LineBreak = "\r\n";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::Initialize(TStrings* AFiles)
{
	try {
		FFiles = new TStringList();
		FFiles->Assign(AFiles);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::Uninitialize()
{
	delete FFiles;
	FFiles = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::MainCreateOutputDir()
{
	if(DirectoryExists(FOutputDir)) {
		return;
	}
	LogInfo("Creates output directory: " + FOutputDir);
	if(!ForceDirectories(FOutputDir)) {
		throw EErrorStop("Cannot create output directory: " + FOutputDir);
	}
	LogInfo("Created output directory: " + FOutputDir);
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::MainConvertGlyphs()
{
	for(int i = 0; i < FFiles->Count; i++) {
		MainEnsureNotCanceled();
		LogInfo("Converts: " + FFiles->Strings[i]);
		ReportStatus(ExtractFileName(FFiles->Strings[i]));
		try {
			TPtr<TGlyphConvertCommand> command = new TGlyphConvertCommand(FFiles->Strings[i], FLightColor, FShadowColor, FOutputDir);
			command->Execute();
			LogInfo("Converted: " + FFiles->Strings[i]);
		}
		catch(EGlyphConvertError& e) {
			LogError(e.Message);
		}
		Progress = MakeProgress(i + 1, FFiles->Count);
		Idle();
	}
	Progress = 100;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::MainEnsureNotCanceled()
{
	if(Canceled) {
		throw EUserCancel("Task canceled by user.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::CreateLog()
{
	try {
		FLogStream = new TFileStream(FLogFile, fmCreate | fmShareDenyWrite);
	}
	catch(EStreamError& e) {
		throw EGlyphConvertTaskError("The program cannot create a log file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::CloseLog()
{
	delete FLogStream;
	FLogStream = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::LogError(const AnsiString& AText)
{
	FErrorCount++;
	LogLine(ErrorPrefix + AText);
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::LogWarning(const AnsiString& AText)
{
	FWarningCount++;
	LogLine(WarningPrefix + AText);
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::LogInfo(const AnsiString& AText)
{
	LogLine(AText);
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::LogLine(const AnsiString& AText)
{
	AnsiString text = AText + LineBreak;
	FLogStream->Write(text.c_str(), text.Length());
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::ReportStatus(const AnsiString& AText)
{
	DoStatus(AText);
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::DoStatus(const AnsiString& AText)
{
	if(OnStatus != NULL) {
		OnStatus(this, AText);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertTask::Main()
{
	try {
		CreateLog();
		LogInfo("Start time: " + FormatDateTime(DateTimeFormat, Now()));
		ReportStatus("Started");
		try {
			MainCreateOutputDir();
			MainConvertGlyphs();
		}
		catch(EUserCancel& e) {
			LogWarning(e.Message);
		}
		catch(EErrorStop& e) {
			LogError(e.Message);
		}
		catch(Exception& e) {
			LogError("An unexpected error occured: " + e.Message);
		}
		catch(...) {
			LogError("An unexpected error occured");
		}
		LogInfo(Progress == 100 ? "Task completed" : "Task canceled");
		LogInfo("Errors: " + AnsiString(ErrorCount));
		LogInfo("Warnings: " + AnsiString(WarningCount));
		LogInfo("End time: " + FormatDateTime(DateTimeFormat, Now()));
		ReportStatus(Progress == 100 ? "Completed" : "Canceled");
	}
	__finally {
		CloseLog();
		FFiles->Clear();
	}
}
//---------------------------------------------------------------------------
__fastcall TGlyphConvertTask::TGlyphConvertTask(TStrings* AFiles, TColor ALightColor, TColor AShadowColor, const AnsiString& AOutputDir, const AnsiString& ALogFile) :
	TTask(tmForeground),
	FFiles(NULL),
	FLightColor(ALightColor),
	FShadowColor(AShadowColor),
	FOutputDir(AOutputDir),
	FLogFile(ALogFile),
	FLogStream(NULL),
	FErrorCount(0),
	FWarningCount(0),
	FOnStatus(NULL)
{
	Initialize(AFiles);
}
//---------------------------------------------------------------------------
__fastcall TGlyphConvertTask::~TGlyphConvertTask()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
}}} // End of Moochos:FullGlyph::GlyphConvertTask
//---------------------------------------------------------------------------

