//---------------------------------------------------------------------------
// GlyphsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "GlyphConvertTask.h"
#include "GlyphsMan.h"
#include "ProgressDialogUnit.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace FullGlyph { namespace GlyphsMan {
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString ConvertTitle = "Convert";
const AnsiString SaveLogAsTitle = "Save Log As";
const AnsiString LogFile = ExtractFilePath(Application->ExeName) + "Log.txt";
const AnsiString LogFileSaveName = "Full Glyph Log.txt";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::Initialize()
{
	try {
		FFilesOpenDialog = new TSaveDialog(NULL);
		FilesOpenDialog->Title = ConvertTitle;
		FilesOpenDialog->Filter = "Bitmaps (*.bmp)|*.bmp|All files|*";
		FilesOpenDialog->Options = FilesOpenDialog->Options << ofAllowMultiSelect;
		FLogSaveDialog = new TSaveDialog(NULL);
		LogSaveDialog->Title = SaveLogAsTitle;
		LogSaveDialog->DefaultExt = "txt";
		LogSaveDialog->Filter = "Text (*.txt)|*.txt|All files|*";
		LogSaveDialog->Options = LogSaveDialog->Options << ofOverwritePrompt;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::Uninitialize()
{
	delete FLogSaveDialog;
	FLogSaveDialog = NULL;

	delete FFilesOpenDialog;
	FFilesOpenDialog = NULL;
}
//---------------------------------------------------------------------------
__fastcall TGlyphsMan::TGlyphsMan() :
	TObject(),
	FTask(NULL),
	FTaskCanceled(false),
	FFilesOpenDialog(NULL),
	FLogSaveDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TGlyphsMan::~TGlyphsMan()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::EnsureAskSaveLog(TGlyphConvertTask* ATask)
{
	try {
		if(ATask->ErrorCount == 0 && ATask->WarningCount == 0) {
			return;
		}
		if(Application->MessageBox(
			AnsiString().sprintf("There are %d errors and %d warnings.\r\n\r\nSave the log right now?", ATask->ErrorCount, ATask->WarningCount).c_str(),
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			return;
		}
		SaveLogAs();
	}
	catch(EGlyphsManError& e) {
		Application->ShowException(&e);
	}
	catch(EAbort& e) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::TaskIdle(TObject* ASender)
{
	if(FTaskCanceled) {
		FTask->Cancel();
	}
	Application->ProcessMessages();
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::TaskProgress(TObject* ASender)
{
	ProgressDialog->ProgressBar->Position = FTask->Progress;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::TaskStatus(TObject* ASender, const AnsiString& AText)
{
	ProgressDialog->MessageLabel->Caption = AText;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::ProgressDialogExecute(TObject* ASender)
{
	FTask->Perform();
	ProgressDialog->CloseDialog();
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::ProgressDialogCancel(TObject* ASender)
{
	if(FTaskCanceled) {
		return;
	}
	if(Application->MessageBox(
		"Cancel the task?",
		Application->Title.c_str(),
		MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
		return;
	}
	FTaskCanceled = true;
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::ChooseFiles(TStrings* AFiles)
{
	try {
		FilesOpenDialog->FileName = AnsiString();
		if(!FilesOpenDialog->Execute()) {
			Abort();
		}
		AFiles->Assign(FilesOpenDialog->Files);
	}
	__finally {
		FilesOpenDialog->FileName = AnsiString();
		FilesOpenDialog->Files->Clear();
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::Convert(TStrings* AFiles, TColor ALightColor, TColor AShadowColor, const AnsiString& AOutputDir)
{
	try {
		try {
			TPtr<TGlyphConvertTask> task = new TGlyphConvertTask(AFiles, ALightColor, AShadowColor, AOutputDir, LogFile);
			FTask = task;
			FTask->OnIdle = TaskIdle;
			FTask->OnProgress = TaskProgress;
			FTask->OnStatus = TaskStatus;
			FTaskCanceled = false;
			ProgressDialog->Caption = ConvertTitle;
			ProgressDialog->MessageLabel->Caption = AnsiString();
			ProgressDialog->NoteLabel->Caption = AnsiString();
			ProgressDialog->ProgressBar->Position = 0;
			ProgressDialog->OnExecute = ProgressDialogExecute;
			ProgressDialog->OnCancel = ProgressDialogCancel;
			ProgressDialog->Execute(true);
			EnsureAskSaveLog(FTask);
		}
		__finally {
			FTask = NULL;
		}
	}
	catch(EGlyphConvertTaskError& e) {
		throw EGlyphsManError(e.Message);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::SaveLogAs()
{
	try {
		LogSaveDialog->FileName = LogFileSaveName;
		if(!LogSaveDialog->Execute()) {
			Abort();
		}
		if(!::CopyFile(LogFile.c_str(), LogSaveDialog->FileName.c_str(), false)) {
			throw EGlyphsManError("The program cannot save the log.");
		}
	}
	__finally {
		LogSaveDialog->FileName = AnsiString();
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphsMan::DeleteLog(bool AConfirm)
{
	if(AConfirm) {
		if(Application->MessageBox(
			"Delete the log?",
			Application->Title.c_str(),
			MB_YESNO | MB_ICONWARNING | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
	if(!::DeleteFile(LogFile.c_str())) {
		throw EGlyphsManError("The program cannot delete the log.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TGlyphsMan::HasLog()
{
	return FileExists(LogFile);
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::FullGlyph::GlyphsMan
//---------------------------------------------------------------------------

